@@ -25,7 +25,7 @@ module Sequel
 
         # H2 uses an IDENTITY type
         def serial_primary_key_options
-          {:primary_key => true, :type => :identity}
+          {:primary_key => true, :type => :identity, :identity=>true}
         end
 
         # H2 supports CREATE TABLE IF NOT EXISTS syntax.
@@ -48,7 +48,7 @@ module Sequel
         # If the :prepare option is given and we aren't in a savepoint,
         # prepare the transaction for a two-phase commit.
         def commit_transaction(conn, opts={})
-          if (s = opts[:prepare]) && @transactions[conn][:savepoint_level] <= 1
+          if (s = opts[:prepare]) && _trans(conn)[:savepoint_level] <= 1
             log_connection_execute(conn, "PREPARE COMMIT #{s}")
           else
             super
@@ -114,6 +114,12 @@ module Sequel
         def schema_column_type(db_type)
           db_type == 'clob' ? :string : super
         end
+
+        # Use BIGINT IDENTITY for identity columns that use bigint, fixes
+        # the case where primary_key :column, :type=>Bignum is used.
+        def type_literal_generic_bignum(column)
+          column[:identity] ? 'BIGINT IDENTITY' : super
+        end
       end
       
       # Dataset class for H2 datasets accessed via JDBC.
@@ -124,7 +130,7 @@ module Sequel
         HSTAR = "H*".freeze
         BITCOMP_OPEN = "((0 - ".freeze
         BITCOMP_CLOSE = ") - 1)".freeze
-        ILIKE_PLACEHOLDER = "CAST(? AS VARCHAR_IGNORECASE)".freeze
+        ILIKE_PLACEHOLDER = ["CAST(".freeze, " AS VARCHAR_IGNORECASE)".freeze].freeze
         TIME_FORMAT = "'%H:%M:%S'".freeze
         
         # Emulate the case insensitive LIKE operator and the bitwise operators.
