@@ -5,11 +5,6 @@ module Sequel
     # Database and Dataset instance methods for JTDS specific
     # support via JDBC.
     module JTDS
-      # Database instance methods for JTDS databases accessed via JDBC.
-      module DatabaseMethods
-        include Sequel::JDBC::MSSQL::DatabaseMethods
-      end
-      
       # Dataset class for JTDS datasets accessed via JDBC.
       class Dataset < JDBC::Dataset
         include Sequel::MSSQL::DatasetMethods
