@@ -1,6 +1,4 @@
 module Sequel
-  Dataset::NON_SQL_OPTIONS << :disable_insert_returning
-
   # Top level module for holding all PostgreSQL-related modules and classes
   # for Sequel.  There are a few module level accessors that are added via
   # metaprogramming.  These are:
@@ -45,25 +43,32 @@ module Sequel
       attr_accessor :client_min_messages
 
       # By default, Sequel forces the use of standard strings, so that
-      # '\\' is interpreted as \\ and not \.  While PostgreSQL defaults
-      # to interpreting plain strings as extended strings, this will change
-      # in a future version of PostgreSQL.  Sequel assumes that SQL standard
-      # strings will be used.
+      # '\\' is interpreted as \\ and not \.  While PostgreSQL <9.1 defaults
+      # to interpreting plain strings, newer versions use standard strings by
+      # default.  Sequel assumes that SQL standard strings will be used.  Setting
+      # this to false means Sequel will use the database's default.
       attr_accessor :force_standard_strings
     end
 
-    # Methods shared by adapter/connection instances.
-    module AdapterMethods
-      attr_writer :db
+    # Methods shared by Database instances that connect to PostgreSQL.
+    module DatabaseMethods
+      EXCLUDE_SCHEMAS = /pg_*|information_schema/i
+      PREPARED_ARG_PLACEHOLDER = LiteralString.new('$').freeze
+      RE_CURRVAL_ERROR = /currval of sequence "(.*)" is not yet defined in this session|relation "(.*)" does not exist/.freeze
+      SYSTEM_TABLE_REGEXP = /^pg|sql/.freeze
+      FOREIGN_KEY_LIST_ON_DELETE_MAP = {'a'.freeze=>:no_action, 'r'.freeze=>:restrict, 'c'.freeze=>:cascade, 'n'.freeze=>:set_null, 'd'.freeze=>:set_default}.freeze
 
-      SELECT_CURRVAL = "SELECT currval('%s')".freeze
-      SELECT_CUSTOM_SEQUENCE = proc do |schema, table| <<-end_sql
-        SELECT '"' || name.nspname || '".' || CASE
+      # SQL fragment for custom sequences (ones not created by serial primary key),
+      # Returning the schema and literal form of the sequence name, by parsing
+      # the column defaults table.
+      SELECT_CUSTOM_SEQUENCE_SQL = (<<-end_sql
+        SELECT name.nspname AS "schema",
+            CASE
             WHEN split_part(def.adsrc, '''', 2) ~ '.' THEN
               substr(split_part(def.adsrc, '''', 2),
                      strpos(split_part(def.adsrc, '''', 2), '.')+1)
             ELSE split_part(def.adsrc, '''', 2)
-          END
+          END AS "sequence"
         FROM pg_class t
         JOIN pg_namespace  name ON (t.relnamespace = name.oid)
         JOIN pg_attribute  attr ON (t.oid = attrelid)
@@ -71,24 +76,26 @@ module Sequel
         JOIN pg_constraint cons ON (conrelid = adrelid AND adnum = conkey[1])
         WHERE cons.contype = 'p'
           AND def.adsrc ~* 'nextval'
-          #{"AND name.nspname = '#{schema}'" if schema}
-          AND t.relname = '#{table}'
       end_sql
-      end
-      SELECT_PK = proc do |schema, table| <<-end_sql
-        SELECT pg_attribute.attname
+      ).strip.gsub(/\s+/, ' ').freeze
+
+      # SQL fragment for determining primary key column for the given table.  Only
+      # returns the first primary key if the table has a composite primary key.
+      SELECT_PK_SQL = (<<-end_sql
+        SELECT pg_attribute.attname AS pk
         FROM pg_class, pg_attribute, pg_index, pg_namespace
         WHERE pg_class.oid = pg_attribute.attrelid
           AND pg_class.relnamespace  = pg_namespace.oid
           AND pg_class.oid = pg_index.indrelid
           AND pg_index.indkey[0] = pg_attribute.attnum
           AND pg_index.indisprimary = 't'
-          #{"AND pg_namespace.nspname = '#{schema}'" if schema}
-          AND pg_class.relname = '#{table}'
       end_sql
-      end
-      SELECT_SERIAL_SEQUENCE = proc do |schema, table| <<-end_sql
-        SELECT  '"' || name.nspname || '".' || seq.relname || ''
+      ).strip.gsub(/\s+/, ' ').freeze
+
+      # SQL fragment for getting sequence associated with table's
+      # primary key, assuming it was a serial primary key column.
+      SELECT_SERIAL_SEQUENCE_SQL = (<<-end_sql
+        SELECT  name.nspname AS "schema", seq.relname AS "sequence"
         FROM pg_class seq, pg_attribute attr, pg_depend dep,
           pg_namespace name, pg_constraint cons
         WHERE seq.oid = dep.objid
@@ -99,68 +106,8 @@ module Sequel
           AND attr.attrelid = cons.conrelid
           AND attr.attnum = cons.conkey[1]
           AND cons.contype = 'p'
-          #{"AND name.nspname = '#{schema}'" if schema}
-          AND seq.relname = '#{table}'
       end_sql
-      end
-
-      # Depth of the current transaction on this connection, used
-      # to implement multi-level transactions with savepoints.
-      attr_accessor :transaction_depth
-
-      # Apply connection settings for this connection. Currently, turns
-      # standard_conforming_strings ON if Postgres.force_standard_strings
-      # is true.
-      def apply_connection_settings
-        if Postgres.force_standard_strings
-          # This setting will only work on PostgreSQL 8.2 or greater
-          # and we don't know the server version at this point, so
-          # try it unconditionally and rescue any errors.
-          execute("SET standard_conforming_strings = ON") rescue nil
-        end
-        if cmm = Postgres.client_min_messages
-          execute("SET client_min_messages = '#{cmm.to_s.upcase}'")
-        end
-      end
-
-      # Get the last inserted value for the given sequence.
-      def last_insert_id(sequence)
-        sql = SELECT_CURRVAL % sequence
-        execute(sql) do |r|
-          val = single_value(r)
-          return val.to_i if val
-        end
-      end
-
-      # Get the primary key for the given table.
-      def primary_key(schema, table)
-        sql = SELECT_PK[schema, table]
-        execute(sql) do |r|
-          return single_value(r)
-        end
-      end
-
-      # Get the primary key and sequence for the given table.
-      def sequence(schema, table)
-        sql = SELECT_SERIAL_SEQUENCE[schema, table]
-        execute(sql) do |r|
-          seq = single_value(r)
-          return seq if seq
-        end
-
-        sql = SELECT_CUSTOM_SEQUENCE[schema, table]
-        execute(sql) do |r|
-          return single_value(r)
-        end
-      end
-    end
-
-    # Methods shared by Database instances that connect to PostgreSQL.
-    module DatabaseMethods
-      EXCLUDE_SCHEMAS = /pg_*|information_schema/i
-      PREPARED_ARG_PLACEHOLDER = LiteralString.new('$').freeze
-      RE_CURRVAL_ERROR = /currval of sequence "(.*)" is not yet defined in this session|relation "(.*)" does not exist/.freeze
-      SYSTEM_TABLE_REGEXP = /^pg|sql/.freeze
+      ).strip.gsub(/\s+/, ' ').freeze
 
       # Commit an existing prepared transaction with the given transaction
       # identifier string.
@@ -266,6 +213,64 @@ module Sequel
         self << drop_trigger_sql(table, name, opts)
       end
 
+      # Return full foreign key information using the pg system tables, including
+      # :name, :on_delete, :on_update, and :deferrable entries in the hashes.
+      def foreign_key_list(table, opts={})
+        m = output_identifier_meth
+        im = input_identifier_meth
+        schema, table = schema_and_table(table)
+        range = 0...32
+
+        base_ds = metadata_dataset.
+          where(:cl__relkind=>'r', :co__contype=>'f', :cl__relname=>im.call(table)).
+          from(:pg_constraint___co).
+          join(:pg_class___cl, :oid=>:conrelid)
+
+        # We split the parsing into two separate queries, which are merged manually later.
+        # This is because PostgreSQL stores both the referencing and referenced columns in
+        # arrays, and I don't know a simple way to not create a cross product, as PostgreSQL
+        # doesn't appear to have a function that takes an array and element and gives you
+        # the index of that element in the array.
+
+        ds = base_ds.
+          join(:pg_attribute___att, :attrelid=>:oid, :attnum=>SQL::Function.new(:ANY, :co__conkey)).
+          order(:co__conname, SQL::CaseExpression.new(range.map{|x| [SQL::Subscript.new(:co__conkey, [x]), x]}, 32, :att__attnum)).
+          select(:co__conname___name, :att__attname___column, :co__confupdtype___on_update, :co__confdeltype___on_delete,
+                 SQL::BooleanExpression.new(:AND, :co__condeferrable, :co__condeferred).as(:deferrable))
+
+        ref_ds = base_ds.
+          join(:pg_class___cl2, :oid=>:co__confrelid).
+          join(:pg_attribute___att2, :attrelid=>:oid, :attnum=>SQL::Function.new(:ANY, :co__confkey)).
+          order(:co__conname, SQL::CaseExpression.new(range.map{|x| [SQL::Subscript.new(:co__conkey, [x]), x]}, 32, :att2__attnum)).
+          select(:co__conname___name, :cl2__relname___table, :att2__attname___refcolumn)
+
+        # If a schema is given, we only search in that schema, and the returned :table
+        # entry is schema qualified as well.
+        if schema
+          ds = ds.join(:pg_namespace___nsp, :oid=>:cl__relnamespace).
+            where(:nsp__nspname=>im.call(schema))
+          ref_ds = ref_ds.join(:pg_namespace___nsp2, :oid=>:cl2__relnamespace).
+            select_more(:nsp2__nspname___schema)
+        end
+
+        h = {}
+        fklod_map = FOREIGN_KEY_LIST_ON_DELETE_MAP 
+        ds.each do |row|
+          if r = h[row[:name]]
+            r[:columns] << m.call(row[:column])
+          else
+            h[row[:name]] = {:name=>m.call(row[:name]), :columns=>[m.call(row[:column])], :on_update=>fklod_map[row[:on_update]], :on_delete=>fklod_map[row[:on_delete]], :deferrable=>row[:deferrable]}
+          end
+        end
+        ref_ds.each do |row|
+          r = h[row[:name]]
+          r[:table] ||= schema ? SQL::QualifiedIdentifier.new(m.call(row[:schema]), m.call(row[:table])) : m.call(row[:table])
+          r[:key] ||= []
+          r[:key] << m.call(row[:refcolumn])
+        end
+        h.values
+      end
+
       # Use the pg_* system tables to determine indexes on a table
       def indexes(table, opts={})
         m = output_identifier_meth
@@ -278,12 +283,11 @@ module Sequel
           join(:pg_index___ind, :indrelid=>:oid, im.call(table)=>:relname).
           join(:pg_class___indc, :oid=>:indexrelid).
           join(:pg_attribute___att, :attrelid=>:tab__oid, :attnum=>attnums).
-          filter(:indc__relkind=>'i', :ind__indisprimary=>false, :indexprs=>nil, :indpred=>nil).
-          order(:indc__relname, range.map{|x| [SQL::Subscript.new(:ind__indkey, [x]), x]}.case(32, :att__attnum)).
+          filter(:indc__relkind=>'i', :ind__indisprimary=>false, :indexprs=>nil, :indpred=>nil, :indisvalid=>true).
+          order(:indc__relname, SQL::CaseExpression.new(range.map{|x| [SQL::Subscript.new(:ind__indkey, [x]), x]}, 32, :att__attnum)).
           select(:indc__relname___name, :ind__indisunique___unique, :att__attname___column)
 
         ds.join!(:pg_namespace___nsp, :oid=>:tab__relnamespace, :nspname=>schema.to_s) if schema
-        ds.filter!(:indisvalid=>true) if server_version >= 80200
         ds.filter!(:indisready=>true, :indcheckxmin=>false) if server_version >= 80300
 
         indexes = {}
@@ -312,32 +316,41 @@ module Sequel
       # Return primary key for the given table.
       def primary_key(table, opts={})
         quoted_table = quote_schema_table(table)
-        return @primary_keys[quoted_table] if @primary_keys.include?(quoted_table)
-        @primary_keys[quoted_table] = if conn = opts[:conn]
-          conn.primary_key(*schema_and_table(table))
-        else
-          synchronize(opts[:server]){|con| con.primary_key(*schema_and_table(table))}
+        @primary_keys.fetch(quoted_table) do
+          schema, table = schema_and_table(table)
+          sql = "#{SELECT_PK_SQL} AND pg_class.relname = #{literal(table)}"
+          sql << "AND pg_namespace.nspname = #{literal(schema)}" if schema
+          @primary_keys[quoted_table] = fetch(sql).single_value
         end
       end
 
       # Return the sequence providing the default for the primary key for the given table.
       def primary_key_sequence(table, opts={})
         quoted_table = quote_schema_table(table)
-        return @primary_key_sequences[quoted_table] if @primary_key_sequences.include?(quoted_table)
-        @primary_key_sequences[quoted_table] = if conn = opts[:conn]
-          conn.sequence(*schema_and_table(table))
-        else
-          synchronize(opts[:server]){|con| con.sequence(*schema_and_table(table))}
+        @primary_key_sequences.fetch(quoted_table) do
+          schema, table = schema_and_table(table)
+          table = literal(table)
+          sql = "#{SELECT_SERIAL_SEQUENCE_SQL} AND seq.relname = #{table}"
+          sql << " AND name.nspname = #{literal(schema)}" if schema
+          unless pks = fetch(sql).single_record
+            sql = "#{SELECT_CUSTOM_SEQUENCE_SQL} AND t.relname = #{table}"
+            sql << " AND name.nspname = #{literal(schema)}" if schema
+            pks = fetch(sql).single_record
+          end
+
+          @primary_key_sequences[quoted_table] = if pks
+            literal(SQL::QualifiedIdentifier.new(pks[:schema], LiteralString.new(pks[:sequence])))
+          end
         end
       end
 
       # Reset the primary key sequence for the given table, baseing it on the
       # maximum current value of the table's primary key.
       def reset_primary_key_sequence(table)
-        pk = SQL::Identifier.new(primary_key(table))
         return unless seq = primary_key_sequence(table)
+        pk = SQL::Identifier.new(primary_key(table))
         db = self
-        seq_ds = db.from(seq.lit)
+        seq_ds = db.from(LiteralString.new(seq))
         get{setval(seq, db[table].select{coalesce(max(pk)+seq_ds.select{:increment_by}, seq_ds.select(:min_value))}, false)}
       end
 
@@ -366,9 +379,20 @@ module Sequel
             0
           end
         end
+        warn 'Sequel no longer supports PostgreSQL <8.2, some things may not work' if @server_version < 80200
         @server_version
       end
 
+      # PostgreSQL supports CREATE TABLE IF NOT EXISTS on 9.1+
+      def supports_create_table_if_not_exists?
+        server_version >= 90100
+      end
+
+      # PostgreSQL supports DROP TABLE IF EXISTS
+      def supports_drop_table_if_exists?
+        true
+      end
+
       # PostgreSQL supports prepared transactions (two-phase commit) if
       # max_prepared_transactions is greater than 0.
       def supports_prepared_transactions?
@@ -376,11 +400,6 @@ module Sequel
         @supports_prepared_transactions = self['SHOW max_prepared_transactions'].get.to_i > 0
       end
 
-      # PostgreSQL supports CREATE TABLE IF NOT EXISTS on 9.1+
-      def supports_create_table_if_not_exists?
-        server_version >= 90100
-      end
-
       # PostgreSQL supports savepoints
       def supports_savepoints?
         true
@@ -391,6 +410,11 @@ module Sequel
         true
       end
 
+      # PostgreSQL supports transaction DDL statements.
+      def supports_transactional_ddl?
+        true
+      end
+
       # Array of symbols specifying table names in the current database.
       # The dataset used is yielded to the block if one is provided,
       # otherwise, an array of symbols of table names is returned.
@@ -402,6 +426,13 @@ module Sequel
         pg_class_relname('r', opts, &block)
       end
 
+      # Check whether the given type name string/symbol (e.g. :hstore) is supported by
+      # the database.
+      def type_supported?(type)
+        @supported_types ||= {}
+        @supported_types.fetch(type){@supported_types[type] = (from(:pg_type).filter(:typtype=>'b', :typname=>type.to_s).count > 0)}
+      end
+
       # Array of symbols specifying view names in the current database.
       #
       # Options:
@@ -413,16 +444,61 @@ module Sequel
 
       private
 
+      # Handle :using option for set_column_type op.
+      def alter_table_sql(table, op)
+        case op[:op]
+        when :set_column_type
+          s = super
+          if using = op[:using]
+            using = Sequel::LiteralString.new(using) if using.is_a?(String)
+            s << ' USING '
+            s << literal(using)
+          end
+          s
+        else
+          super
+        end
+      end
+
+      # If the :synchronous option is given and non-nil, set synchronous_commit
+      # appropriately.  Valid values for the :synchronous option are true,
+      # :on, false, :off, :local, and :remote_write.
+      def begin_new_transaction(conn, opts)
+        super
+        if opts.has_key?(:synchronous)
+          case sync = opts[:synchronous]
+          when true
+            sync = :on
+          when false
+            sync = :off
+          when nil
+            return
+          end
+
+          log_connection_execute(conn, "SET LOCAL synchronous_commit = #{sync}")
+        end
+      end
+      
       # If the :prepare option is given and we aren't in a savepoint,
       # prepare the transaction for a two-phase commit.
       def commit_transaction(conn, opts={})
-        if (s = opts[:prepare]) && @transactions[conn][:savepoint_level] <= 1
+        if (s = opts[:prepare]) && _trans(conn)[:savepoint_level] <= 1
           log_connection_execute(conn, "PREPARE TRANSACTION #{literal(s)}")
         else
           super
         end
       end
 
+      # The SQL queries to execute when starting a new connection.
+      def connection_configuration_sqls
+        sqls = []
+        sqls << "SET standard_conforming_strings = ON" if Postgres.force_standard_strings
+        if cmm = Postgres.client_min_messages
+          sqls << "SET client_min_messages = '#{cmm.to_s.upcase}'"
+        end
+        sqls
+      end
+
       # SQL statement to create database function.
       def create_function_sql(name, definition, opts={})
         args = opts[:args]
@@ -470,6 +546,11 @@ module Sequel
       def drop_function_sql(name, opts={})
         "DROP FUNCTION#{' IF EXISTS' if opts[:if_exists]} #{name}#{sql_function_args(opts[:args])}#{' CASCADE' if opts[:cascade]}"
       end
+      
+      # Support :if_exists, :cascade, and :concurrently options.
+      def drop_index_sql(table, op)
+        "DROP INDEX#{' CONCURRENTLY' if op[:concurrently]}#{' IF EXISTS' if op[:if_exists]} #{quote_identifier(op[:name] || default_index_name(table, op[:columns]))}#{' CASCADE' if op[:cascade]}"
+      end
 
       # SQL for dropping a procedural language from the database.
       def drop_language_sql(name, opts={})
@@ -521,45 +602,12 @@ module Sequel
         filter = " WHERE #{filter_expr(filter)}" if filter
         case index_type
         when :full_text
-          expr = "(to_tsvector(#{literal(index[:language] || 'simple')}, #{literal(dataset.send(:full_text_string_join, cols))}))"
+          expr = "(to_tsvector(#{literal(index[:language] || 'simple')}::regconfig, #{literal(dataset.send(:full_text_string_join, cols))}))"
           index_type = :gin
         when :spatial
           index_type = :gist
         end
-        "CREATE #{unique}INDEX #{quote_identifier(index_name)} ON #{quote_schema_table(table_name)} #{"USING #{index_type} " if index_type}#{expr}#{filter}"
-      end
-
-      # The result of the insert for the given table and values.  If values
-      # is an array, assume the first column is the primary key and return
-      # that.  If values is a hash, lookup the primary key for the table.  If
-      # the primary key is present in the hash, return its value.  Otherwise,
-      # look up the sequence for the table's primary key.  If one exists,
-      # return the last value the of the sequence for the connection.
-      def insert_result(conn, table, values)
-        case values
-        when Hash
-          return nil unless pk = primary_key(table, :conn=>conn)
-          if pk and pkv = values[pk.to_sym]
-            pkv
-          else
-            begin
-              if seq = primary_key_sequence(table, :conn=>conn)
-                conn.last_insert_id(seq)
-              end
-            rescue Exception => e
-              raise_error(e, :classes=>CONVERTED_EXCEPTIONS) unless RE_CURRVAL_ERROR.match(e.message)
-            end
-          end
-        when Array
-          values.first
-        else
-          nil
-        end
-      end
-
-      # Don't log, since logging is done by the underlying connection.
-      def log_connection_execute(conn, sql)
-        conn.execute(sql)
+        "CREATE #{unique}INDEX#{' CONCURRENTLY' if index[:concurrently]} #{quote_identifier(index_name)} ON #{quote_schema_table(table_name)} #{"USING #{index_type} " if index_type}#{expr}#{filter}"
       end
 
       # Backbone of the tables and views support.
@@ -634,6 +682,20 @@ module Sequel
         end
       end
 
+      # Set the transaction isolation level on the given connection
+      def set_transaction_isolation(conn, opts)
+        level = opts.fetch(:isolation, transaction_isolation_level)
+        read_only = opts[:read_only]
+        deferrable = opts[:deferrable]
+        if level || !read_only.nil? || !deferrable.nil?
+          sql = "SET TRANSACTION"
+          sql << " ISOLATION LEVEL #{Sequel::Database::TRANSACTION_ISOLATION_LEVELS[level]}" if level
+          sql << " READ #{read_only ? 'ONLY' : 'WRITE'}" unless read_only.nil?
+          sql << " #{'NOT ' unless deferrable}DEFERRABLE" unless deferrable.nil?
+          log_connection_execute(conn, sql)
+        end
+      end
+     
       # Turns an array of argument specifiers into an SQL fragment used for function arguments.  See create_function_sql.
       def sql_function_args(args)
         "(#{Array(args).map{|a| Array(a).reverse.join(' ')}.join(', ')})"
@@ -676,14 +738,13 @@ module Sequel
       BOOL_FALSE = 'false'.freeze
       BOOL_TRUE = 'true'.freeze
       COMMA_SEPARATOR = ', '.freeze
-      DELETE_CLAUSE_METHODS = Dataset.clause_methods(:delete, %w'delete from using where')
+      DELETE_CLAUSE_METHODS = Dataset.clause_methods(:delete, %w'delete from using where returning')
       DELETE_CLAUSE_METHODS_91 = Dataset.clause_methods(:delete, %w'with delete from using where returning')
       EXCLUSIVE = 'EXCLUSIVE'.freeze
       EXPLAIN = 'EXPLAIN '.freeze
       EXPLAIN_ANALYZE = 'EXPLAIN ANALYZE '.freeze
       FOR_SHARE = ' FOR SHARE'.freeze
-      INSERT_CLAUSE_METHODS = Dataset.clause_methods(:insert, %w'insert into columns values')
-      INSERT_CLAUSE_METHODS_82 = Dataset.clause_methods(:insert, %w'insert into columns values returning')
+      INSERT_CLAUSE_METHODS = Dataset.clause_methods(:insert, %w'insert into columns values returning')
       INSERT_CLAUSE_METHODS_91 = Dataset.clause_methods(:insert, %w'with insert into columns values returning')
       LOCK = 'LOCK TABLE %s IN %s MODE'.freeze
       NULL = LiteralString.new('NULL').freeze
@@ -697,7 +758,7 @@ module Sequel
       SHARE_ROW_EXCLUSIVE = 'SHARE ROW EXCLUSIVE'.freeze
       SHARE_UPDATE_EXCLUSIVE = 'SHARE UPDATE EXCLUSIVE'.freeze
       SQL_WITH_RECURSIVE = "WITH RECURSIVE ".freeze
-      UPDATE_CLAUSE_METHODS = Dataset.clause_methods(:update, %w'update table set from where')
+      UPDATE_CLAUSE_METHODS = Dataset.clause_methods(:update, %w'update table set from where returning')
       UPDATE_CLAUSE_METHODS_91 = Dataset.clause_methods(:update, %w'with update table set from where returning')
       SPACE = Dataset::SPACE
       FROM = Dataset::FROM
@@ -717,8 +778,8 @@ module Sequel
       # Shared methods for prepared statements when used with PostgreSQL databases.
       module PreparedStatementMethods
         # Override insert action to use RETURNING if the server supports it.
-        def run(&block)
-          if @prepared_type == :insert && supports_insert_select?
+        def run
+          if @prepared_type == :insert
             fetch_rows(prepared_sql){|r| return r.values.first}
           else
             super
@@ -727,23 +788,13 @@ module Sequel
 
         def prepared_sql
           return @prepared_sql if @prepared_sql
-          @opts[:returning] = insert_pk if @prepared_type == :insert && supports_insert_select?
+          @opts[:returning] = insert_pk if @prepared_type == :insert
           super
           @prepared_sql
         end
       end
 
-      # Add the disable_insert_returning! mutation method
-      def self.extended(obj)
-        obj.def_mutation_method(:disable_insert_returning)
-      end
-
-      # Add the disable_insert_returning! mutation method
-      def self.included(mod)
-        mod.def_mutation_method(:disable_insert_returning)
-      end
-
-      # Return the results of an ANALYZE query as a string
+      # Return the results of an EXPLAIN ANALYZE query as a string
       def analyze
         explain(:analyze=>true)
       end
@@ -765,11 +816,6 @@ module Sequel
         end
       end
 
-      # Disable the use of INSERT RETURNING, even if the server supports it
-      def disable_insert_returning
-        clone(:disable_insert_returning=>true)
-      end
-
       # Return the results of an EXPLAIN query as a string
       def explain(opts={})
         with_sql((opts[:analyze] ? EXPLAIN_ANALYZE : EXPLAIN) + select_sql).map(QUERY_PLAN).join(CRLF)
@@ -785,32 +831,29 @@ module Sequel
       def full_text_search(cols, terms, opts = {})
         lang = opts[:language] || 'simple'
         terms = terms.join(' | ') if terms.is_a?(Array)
-        filter("to_tsvector(?, ?) @@ to_tsquery(?, ?)", lang, full_text_string_join(cols), lang, terms)
+        filter("to_tsvector(?::regconfig, ?) @@ to_tsquery(?::regconfig, ?)", lang, full_text_string_join(cols), lang, terms)
       end
 
       # Insert given values into the database.
-      def insert(*values, &block)
+      def insert(*values)
         if @opts[:returning]
+          # already know which columns to return, let the standard code
+          # handle it
           super
-        elsif !@opts[:sql] && supports_insert_select?
-          returning(insert_pk).insert(*values){|r| return r.values.first}
-        elsif (f = opts[:from]) && !f.empty?
-          v = if values.size == 1
-            values.first
-          elsif values.size == 2 && values.all?{|v0| v0.is_a?(Array)}
-            Hash[*values.first.zip(values.last).flatten]
-          else
-            values
-          end
-          execute_insert(insert_sql(*values), :table=>f.first, :values=>v)
+        elsif @opts[:sql]
+          # raw SQL used, so don't know which table is being inserted
+          # into, and therefore can't determine primary key.  Run the
+          # insert statement and return nil.
+          super
+          nil
         else
-          super
+          # Force the use of RETURNING with the primary key value.
+          returning(insert_pk).insert(*values){|r| return r.values.first}
         end
       end
 
       # Insert a record returning the record inserted
       def insert_select(*values)
-        return unless supports_insert_select?
         returning.insert(*values){|r| return r}
       end
 
@@ -828,11 +871,8 @@ module Sequel
         nil
       end
 
-      # For PostgreSQL version > 8.2, allow inserting multiple rows at once.
+      # PostgreSQL allows inserting multiple rows at once.
       def multi_insert_sql(columns, values)
-        return super if server_version < 80200
-
-        # postgresql 8.2 introduces support for multi-row insert
         sql = LiteralString.new('VALUES ')
         expression_list_append(sql, values.map{|r| Array(r)})
         [insert_sql(columns, sql)]
@@ -854,12 +894,9 @@ module Sequel
         true
       end
 
+      # Returning is always supported.
       def supports_returning?(type)
-        if type == :insert
-          server_version >= 80200 && !opts[:disable_insert_returning]
-        else
-          server_version >= 80200
-        end
+        true
       end
 
       # PostgreSQL supports timezones in literal timestamps
@@ -872,6 +909,29 @@ module Sequel
         server_version >= 80400
       end
 
+      # Truncates the dataset.  Returns nil.
+      #
+      # Options:
+      # :cascade :: whether to use the CASCADE option, useful when truncating
+      #   tables with Foreign Keys.
+      # :only :: truncate using ONLY, so child tables are unaffected
+      # :restart :: use RESTART IDENTITY to restart any related sequences
+      #
+      # :only and :restart only work correctly on PostgreSQL 8.4+.
+      #
+      # Usage:
+      #   DB[:table].truncate # TRUNCATE TABLE "table"
+      #   # => nil
+      #   DB[:table].truncate(:cascade => true, :only=>true, :restart=>true) # TRUNCATE TABLE ONLY "table" RESTART IDENTITY CASCADE
+      #   # => nil
+      def truncate(opts = {})
+        if opts.empty?
+          super()
+        else
+          clone(:truncate_opts=>opts).truncate
+        end
+      end
+
       # Return a clone of the dataset with an addition named window that can be referenced in window functions.
       def window(name, opts)
         clone(:window=>(@opts[:window]||[]) + [[name, SQL::Window.new(opts)]])
@@ -884,17 +944,13 @@ module Sequel
       # is only set to return a single columns, return an array of just that column.
       # Otherwise, return an array of hashes.
       def _import(columns, values, opts={})
-        if server_version >= 80200
-          if opts[:return] == :primary_key && !@opts[:returning]
-            returning(insert_pk)._import(columns, values, opts)
-          elsif @opts[:returning]
-            statements = multi_insert_sql(columns, values)
-            @db.transaction(opts.merge(:server=>@opts[:server])) do
-              statements.map{|st| returning_fetch_rows(st)}
-            end.first.map{|v| v.length == 1 ? v.values.first : v}
-          else
-            super
-          end
+        if @opts[:returning]
+          statements = multi_insert_sql(columns, values)
+          @db.transaction(opts.merge(:server=>@opts[:server])) do
+            statements.map{|st| returning_fetch_rows(st)}
+          end.first.map{|v| v.length == 1 ? v.values.first : v}
+        elsif opts[:return] == :primary_key
+          returning(insert_pk)._import(columns, values, opts)
         else
           super
         end
@@ -902,9 +958,25 @@ module Sequel
 
       private
 
+      # Format TRUNCATE statement with PostgreSQL specific options.
+      def _truncate_sql(table)
+        to = @opts[:truncate_opts] || {}
+        "TRUNCATE TABLE#{' ONLY' if to[:only]} #{table}#{' RESTART IDENTITY' if to[:restart]}#{' CASCADE' if to[:cascade]}"
+      end
+
+      # Allow truncation of multiple source tables.
+      def check_truncation_allowed!
+        raise(InvalidOperation, "Grouped datasets cannot be truncated") if opts[:group]
+        raise(InvalidOperation, "Joined datasets cannot be truncated") if opts[:join]
+      end
+
       # PostgreSQL allows deleting from joined datasets
       def delete_clause_methods
-        server_version >= 90100 ? DELETE_CLAUSE_METHODS_91 : DELETE_CLAUSE_METHODS
+        if server_version >= 90100
+          DELETE_CLAUSE_METHODS_91
+        else
+          DELETE_CLAUSE_METHODS
+        end
       end
 
       # Only include the primary table in the main delete clause
@@ -922,8 +994,6 @@ module Sequel
       def insert_clause_methods
         if server_version >= 90100
           INSERT_CLAUSE_METHODS_91
-        elsif server_version >= 80200
-          INSERT_CLAUSE_METHODS_82
         else
           INSERT_CLAUSE_METHODS
         end
@@ -931,8 +1001,9 @@ module Sequel
 
       # Return the primary key to use for RETURNING in an INSERT statement
       def insert_pk
-        pk = db.primary_key(opts[:from].first) if opts[:from] && !opts[:from].empty?
-        pk ? Sequel::SQL::Identifier.new(pk) : NULL
+        if (f = opts[:from]) && !f.empty? && (pk = db.primary_key(f.first))
+          Sequel::SQL::Identifier.new(pk)
+        end
       end
 
       # For multiple table support, PostgreSQL requires at least
@@ -1035,7 +1106,11 @@ module Sequel
 
       # PostgreSQL splits the main table from the joined tables
       def update_clause_methods
-        server_version >= 90100 ? UPDATE_CLAUSE_METHODS_91 : UPDATE_CLAUSE_METHODS
+        if server_version >= 90100
+          UPDATE_CLAUSE_METHODS_91
+        else
+          UPDATE_CLAUSE_METHODS
+        end
       end
 
       # Use FROM to specify additional tables in an update query
