@@ -28,12 +28,6 @@ module Sequel
         pragma_set(:auto_vacuum, value)
       end
 
-      # Boolean signifying the value of the case_sensitive_likePRAGMA, or nil
-      # if not using SQLite 3.2.3+.
-      def case_sensitive_like
-        pragma_get(:case_sensitive_like).to_i == 1 if sqlite_version >= 30203
-      end
-      
       # Set the case_sensitive_like PRAGMA using the given boolean value, if using
       # SQLite 3.2.3+.  If not using 3.2.3+, no error is raised. See pragma_set.
       # Consider using the :case_sensitive_like Database option instead.
@@ -59,22 +53,33 @@ module Sequel
         pragma_set(:foreign_keys, !!value ? 'on' : 'off') if sqlite_version >= 30619
       end
 
+      # Return the array of foreign key info hashes using the foreign_key_list PRAGMA,
+      # including information for the :on_update and :on_delete entries.
+      def foreign_key_list(table, opts={})
+        m = output_identifier_meth
+        h = {}
+        metadata_dataset.with_sql("PRAGMA foreign_key_list(?)", input_identifier_meth.call(table)).each do |row|
+          if r = h[row[:id]]
+            r[:columns] << m.call(row[:from])
+            r[:key] << m.call(row[:to]) if r[:key]
+          else
+            h[row[:id]] = {:columns=>[m.call(row[:from])], :table=>m.call(row[:table]), :key=>([m.call(row[:to])] if row[:to]), :on_update=>on_delete_sql_to_sym(row[:on_update]), :on_delete=>on_delete_sql_to_sym(row[:on_delete])}
+          end
+        end
+        h.values
+      end
+
       # Use the index_list and index_info PRAGMAs to determine the indexes on the table.
       def indexes(table, opts={})
         m = output_identifier_meth
         im = input_identifier_meth
         indexes = {}
-        begin
-          metadata_dataset.with_sql("PRAGMA index_list(?)", im.call(table)).each do |r|
-            next if r[:name] =~ PRIMARY_KEY_INDEX_RE
-            indexes[m.call(r[:name])] = {:unique=>r[:unique].to_i==1}
-          end
-        rescue Sequel::DatabaseError
-          nil
-        else
-          indexes.each do |k, v|
-            v[:columns] = metadata_dataset.with_sql("PRAGMA index_info(?)", im.call(k)).map(:name).map{|x| m.call(x)}
-          end
+        metadata_dataset.with_sql("PRAGMA index_list(?)", im.call(table)).each do |r|
+          next if r[:name] =~ PRIMARY_KEY_INDEX_RE
+          indexes[m.call(r[:name])] = {:unique=>r[:unique].to_i==1}
+        end
+        indexes.each do |k, v|
+          v[:columns] = metadata_dataset.with_sql("PRAGMA index_info(?)", im.call(k)).map(:name).map{|x| m.call(x)}
         end
         indexes
       end
@@ -226,7 +231,7 @@ module Sequel
           when :primary_key
             duplicate_table(table){|columns| columns.each{|s| s[:primary_key] = nil}}
           when :foreign_key
-            duplicate_table(table){|columns| columns.each{|s| s[:table] = nil}}
+            duplicate_table(table, :no_foreign_keys=>true)
           when :unique
             duplicate_table(table)
           else
@@ -241,7 +246,7 @@ module Sequel
         end
       end
 
-      # The array of column symbols in the table, except for ones given in opts[:except]
+      # A name to use for the backup table
       def backup_table_name(table, opts={})
         table = table.gsub('`', '')
         (opts[:times]||1000).times do |i|
@@ -272,31 +277,13 @@ module Sequel
         ps
       end
 
-      # The array of column schema hashes, except for the ones given in opts[:except]
-      def defined_columns_for(table, opts={})
+      # The array of column schema hashes for the current columns in the table
+      def defined_columns_for(table)
         cols = parse_pragma(table, {})
         cols.each do |c|
           c[:default] = LiteralString.new(c[:default]) if c[:default]
           c[:type] = c[:db_type]
         end
-        if opts[:except]
-          nono= Array(opts[:except]).compact.map{|n| n.to_s}
-          cols.reject!{|c| nono.include? c[:name] }
-        end
-
-        begin
-          metadata_dataset.with_sql("PRAGMA foreign_key_list(?)", input_identifier_meth.call(table)).each do |row|
-            c = cols.find {|co| co[:name] == row[:from] } or next
-            c[:table] = row[:table]
-            c[:key] = row[:to]
-            c[:on_update] = on_delete_sql_to_sym(row[:on_update])
-            c[:on_delete] = on_delete_sql_to_sym(row[:on_delete])
-            # is there any way to get deferrable status?
-          end
-        rescue Sequel::DatabaseError
-          # Doesn't work correctly on some versions of JDBC SQLite,
-          # giving a "query does not return ResultSet" error.
-        end
         cols
       end
 
@@ -319,6 +306,20 @@ module Sequel
           def_columns.each{|c| c[:primary_key] = false if c[:primary_key]}
         end
 
+        # If dropping a foreign key constraint, drop all foreign key constraints,
+        # as there is no way to determine which one to drop.
+        unless opts[:no_foreign_keys]
+          fks = foreign_key_list(table)
+
+          # If dropping a column, if there is a foreign key with that
+          # column, don't include it when building a copy of the table.
+          if ocp = opts[:old_columns_proc]
+            fks.delete_if{|c| ocp.call(c[:columns].dup) != c[:columns]}
+          end
+
+          constraints.concat(fks.each{|h| h[:type] = :foreign_key})
+        end
+
         def_columns_str = (def_columns.map{|c| column_definition_sql(c)} + constraints.map{|c| constraint_definition_sql(c)}).join(', ')
         new_columns = old_columns.dup
         opts[:new_columns_proc].call(new_columns) if opts[:new_columns_proc]
@@ -399,13 +400,12 @@ module Sequel
         m = output_identifier_meth
         metadata_dataset.from(:sqlite_master).server(opts[:server]).filter(filter).map{|r| m.call(r[:name])}
       end
-      
-      # SQLite uses the integer data type even for bignums.  This is because they
-      # are both stored internally as text, and converted when returned from
-      # the database.  Using an integer type instead of bigint makes it more likely
-      # that software will automatically return the column as an integer.
+
+      # SQLite only supports AUTOINCREMENT on integer columns, not
+      # bigint columns, so use integer instead of bigint for those
+      # columns.
       def type_literal_generic_bignum(column)
-        :integer
+        column[:auto_increment] ? :integer : super
       end
     end
     
@@ -424,6 +424,8 @@ module Sequel
       NUMERIC = 'NUMERIC'.freeze
       INTEGER = 'INTEGER'.freeze
       BACKTICK = '`'.freeze
+      BACKTICK_RE = /`/.freeze
+      DOUBLE_BACKTICK = '``'.freeze
       BLOB_START = "X'".freeze
       HSTAR = "H*".freeze
 
@@ -470,14 +472,19 @@ module Sequel
       # Since we want to always return the count of records, add a condition
       # that is always true and then delete.
       def delete
-        @opts[:where] ? super : filter(1=>1).delete
+        @opts[:where] ? super : where(1=>1).delete
       end
       
       # Return an array of strings specifying a query explanation for a SELECT of the
-      # current dataset.
-      def explain
-        db.send(:metadata_dataset).clone(:sql=>"EXPLAIN #{select_sql}").
-          map{|x| "#{x[:addr]}|#{x[:opcode]}|#{(1..5).map{|i| x[:"p#{i}"]}.join('|')}|#{x[:comment]}"}
+      # current dataset. Currently, the options are ignore, but it accepts options
+      # to be compatible with other adapters.
+      def explain(opts=nil)
+        # Load the PrettyTable class, needed for explain output
+        Sequel.extension(:_pretty_table) unless defined?(Sequel::PrettyTable)
+
+        ds = db.send(:metadata_dataset).clone(:sql=>"EXPLAIN #{select_sql}")
+        rows = ds.all
+        Sequel::PrettyTable.string(rows, ds.columns)
       end
       
       # HAVING requires GROUP BY on SQLite
@@ -488,7 +495,7 @@ module Sequel
       
       # SQLite uses the nonstandard ` (backtick) for quoting identifiers.
       def quoted_identifier_append(sql, c)
-        sql << BACKTICK << c.to_s << BACKTICK
+        sql << BACKTICK << c.to_s.gsub(BACKTICK_RE, DOUBLE_BACKTICK) << BACKTICK
       end
       
       # When a qualified column is selected on SQLite and the qualifier
