@@ -144,11 +144,6 @@ module Sequel
         _execute(:insert, sql, opts)
       end
       
-      # Run the given SQL with the given arguments and return the first value of the first row.
-      def single_value(sql, opts={})
-        _execute(:single_value, sql, opts)
-      end
-      
       # Handle Integer and Float arguments, since SQLite can store timestamps as integers and floats.
       def to_application_timestamp(s)
         case s
@@ -177,8 +172,6 @@ module Sequel
             case type
             when :select
               log_yield(sql, log_args){conn.query(sql, args, &block)}
-            when :single_value
-              log_yield(sql, log_args){conn.get_first_value(sql, args)}
             when :insert
               log_yield(sql, log_args){conn.execute(sql, args)}
               conn.last_insert_row_id
@@ -216,7 +209,7 @@ module Sequel
 
       # Execute a prepared statement on the database using the given name.
       def execute_prepared_statement(conn, type, name, opts, &block)
-        ps = prepared_statements[name]
+        ps = prepared_statement(name)
         sql = ps.prepared_sql
         args = opts[:arguments]
         ps_args = {}
@@ -229,13 +222,19 @@ module Sequel
           end
         end
         unless cps
-          cps = log_yield("Preparing #{name}: #{sql}"){conn.prepare(sql)}
+          cps = log_yield("PREPARE #{name}: #{sql}"){conn.prepare(sql)}
           conn.prepared_statements[name] = [cps, sql]
         end
+        log_sql = "EXECUTE #{name}"
+        if ps.log_sql
+          log_sql << " ("
+          log_sql << sql
+          log_sql << ")"
+        end
         if block
-          log_yield("Executing prepared statement #{name}", args){cps.execute(ps_args, &block)}
+          log_yield(log_sql, args){cps.execute(ps_args, &block)}
         else
-          log_yield("Executing prepared statement #{name}", args){cps.execute!(ps_args){|r|}}
+          log_yield(log_sql, args){cps.execute!(ps_args){|r|}}
           case type
           when :insert
             conn.last_insert_row_id
@@ -376,7 +375,7 @@ module Sequel
         ps.extend(PreparedStatementMethods)
         if name
           ps.prepared_statement_name = name
-          db.prepared_statements[name] = ps
+          db.set_prepared_statement(name, ps)
         end
         ps
       end
