@@ -44,9 +44,9 @@ module Sequel
       descendants << base
     end
 
-    # Always use transactions for old-style migrations.
+    # Don't allow transaction overriding in old migrations.
     def self.use_transactions
-      true
+      nil
     end
     
     # The default down action does nothing
@@ -74,12 +74,13 @@ module Sequel
     # Proc used for the up action
     attr_accessor :up
 
-    # Whether to use transactions for this migration, true by default.
+    # Whether to use transactions for this migration, default depends on the
+    # database.
     attr_accessor :use_transactions
 
-    # Use transactions by default
+    # Don't set transaction use by default.
     def initialize
-      @use_transactions = true
+      @use_transactions = nil
     end
 
     # Apply the appropriate block on the +Database+
@@ -118,6 +119,11 @@ module Sequel
       migration.use_transactions = false
     end
 
+    # Enable the use of transactions for the related migration
+    def transaction
+      migration.use_transactions = true
+    end
+
     # Defines the migration's up action.
     def up(&block)
       migration.up = block
@@ -184,6 +190,10 @@ module Sequel
       @actions << [:alter_table, table, MigrationAlterTableReverser.new.reverse(&block)]
     end
 
+    def create_join_table(*args)
+      @actions << [:drop_join_table, *args]
+    end
+
     def create_table(*args)
       @actions << [:drop_table, args.first]
     end
@@ -330,11 +340,28 @@ module Sequel
     class Error < Sequel::Error
     end
 
+    # Exception class raised when Migrator.check_current signals that it is
+    # not current.
+    class NotCurrentError < Error
+    end
+
     # Wrapper for +run+, maintaining backwards API compatibility
     def self.apply(db, directory, target = nil, current = nil)
       run(db, directory, :target => target, :current => current)
     end
 
+    # Raise a NotCurrentError unless the migrator is current, takes the same
+    # arguments as #run.
+    def self.check_current(*args)
+      raise(NotCurrentError, 'migrator is not current') unless is_current?(*args)
+    end
+
+    # Return whether the migrator is current (i.e. it does not need to make
+    # any changes).  Takes the same arguments as #run.
+    def self.is_current?(db, directory, opts={})
+      migrator_class(directory).new(db, directory, opts).is_current?
+    end
+
     # Migrates the supplied database using the migration files in the the specified directory. Options:
     # * :column :: The column in the :table argument storing the migration version (default: :version).
     # * :current :: The current version of the database.  If not given, it is retrieved from the database
@@ -410,9 +437,17 @@ module Sequel
     # If transactions should be used for the migration, yield to the block
     # inside a transaction.  Otherwise, just yield to the block.
     def checked_transaction(migration, &block)
-      if @use_transactions == false
-        yield
-      elsif migration.use_transactions || @use_transactions
+      use_trans = if @use_transactions.nil?
+        if migration.use_transactions.nil?
+          @db.supports_transactional_ddl?
+        else
+          migration.use_transactions
+        end
+      else
+        @use_transactions
+      end
+
+      if use_trans
         db.transaction(&block)
       else
         yield
@@ -466,6 +501,11 @@ module Sequel
       @migrations = get_migrations
     end
 
+    # The integer migrator is current if the current version is the same as the target version.
+    def is_current?
+      current_migration_version == target
+    end
+    
     # Apply all migrations on the database
     def run
       migrations.zip(version_numbers).each do |m, v|
@@ -583,6 +623,12 @@ module Sequel
       @migration_tuples = get_migration_tuples
     end
 
+    # The timestamp migrator is current if there are no migrations to apply
+    # in either direction.
+    def is_current?
+      migration_tuples.empty?
+    end
+    
     # Apply all migration tuples on the database
     def run
       migration_tuples.each do |m, f, direction|
