@@ -26,9 +26,7 @@ def logger.method_missing(m, msg)
   MYSQL_DB.sqls << msg
 end
 MYSQL_DB.loggers = [logger]
-MYSQL_DB.drop_table(:items) rescue nil
-MYSQL_DB.drop_table(:dolls) rescue nil
-MYSQL_DB.drop_table(:booltest) rescue nil
+MYSQL_DB.drop_table?(:items, :dolls, :booltest)
 
 SQL_BEGIN = 'BEGIN'
 SQL_ROLLBACK = 'ROLLBACK'
@@ -40,7 +38,7 @@ describe "MySQL", '#create_table' do
     MYSQL_DB.sqls.clear
   end
   after do
-    @db.drop_table(:dolls) rescue nil
+    @db.drop_table?(:dolls)
   end
 
   specify "should allow to specify options for MySQL" do
@@ -83,7 +81,7 @@ describe "MySQL", '#create_table' do
     @db.schema(:dolls).map{|k, v| v[:auto_increment]}.should == [nil, nil, true]
   end
 
-  specify "should support collate with various other column options" do
+  cspecify "should support collate with various other column options", :swift do
     @db.create_table!(:dolls){ String :name, :size=>128, :collate=>:utf8_bin, :default=>'foo', :null=>false, :unique=>true}
     @db[:dolls].insert
     @db[:dolls].select_map(:name).should == ["foo"]
@@ -113,7 +111,7 @@ if MYSQL_DB.adapter_scheme == :mysql
     end
     after do
       @db.convert_tinyint_to_bool = true
-      @db.drop_table(:booltest)
+      @db.drop_table?(:booltest)
     end
 
     specify "should consider tinyint(1) datatypes as boolean if set, but not larger tinyints" do
@@ -161,7 +159,7 @@ describe "A MySQL dataset" do
     MYSQL_DB.sqls.clear
   end
   after do
-    MYSQL_DB.drop_table(:items)
+    MYSQL_DB.drop_table?(:items)
   end
 
   specify "should quote columns and tables using back-ticks if quoting identifiers" do
@@ -241,6 +239,12 @@ describe "A MySQL dataset" do
     @d.filter(:name => /^bc/).count.should == 1
   end
 
+  specify "should have explain output" do
+    @d.explain.should be_a_kind_of(String)
+    @d.explain(:extended=>true).should be_a_kind_of(String)
+    @d.explain.should_not == @d.explain(:extended=>true)
+  end
+
   specify "should correctly literalize strings with comment backslashes in them" do
     @d.delete
     proc {@d << {:name => ':\\'}}.should_not raise_error
@@ -285,7 +289,7 @@ describe "Dataset#distinct" do
     @ds = @db[:a]
   end
   after do
-    @db.drop_table(:a)
+    @db.drop_table?(:a)
   end
 
   it "#distinct with arguments should return results distinct on those arguments" do
@@ -450,12 +454,12 @@ describe "A MySQL database with table options" do
     Sequel::MySQL.default_collate = 'utf8_general_ci'
 
     @db = MYSQL_DB
-    @db.drop_table(:items) rescue nil
+    @db.drop_table?(:items)
 
     MYSQL_DB.sqls.clear
   end
   after do
-    @db.drop_table(:items) rescue nil
+    @db.drop_table?(:items)
 
     Sequel::MySQL.default_engine = nil
     Sequel::MySQL.default_charset = nil
@@ -481,12 +485,11 @@ end
 describe "A MySQL database" do
   before do
     @db = MYSQL_DB
-    @db.drop_table(:items) rescue nil
+    @db.drop_table?(:items)
     MYSQL_DB.sqls.clear
   end
   after do
-    @db.drop_table(:items) rescue nil
-    @db.drop_table(:users) rescue nil
+    @db.drop_table?(:items, :users)
   end
 
   specify "should support defaults for boolean columns" do
@@ -601,8 +604,7 @@ end
 
 describe "MySQL foreign key support" do
   after do
-    MYSQL_DB.drop_table(:testfk) rescue nil
-    MYSQL_DB.drop_table(:testpk) rescue nil
+    MYSQL_DB.drop_table?(:testfk, :testpk)
   end
 
   specify "should create table without :key" do
@@ -674,11 +676,11 @@ end
 describe "A MySQL database" do
   before do
     @db = MYSQL_DB
-    @db.drop_table(:posts) rescue nil
+    @db.drop_table?(:posts)
     @db.sqls.clear
   end
   after do
-    @db.drop_table(:posts) rescue nil
+    @db.drop_table?(:posts)
   end
 
   specify "should support fulltext indexes and full_text_search" do
@@ -750,7 +752,7 @@ describe "MySQL::Dataset#insert and related methods" do
     MYSQL_DB.sqls.clear
   end
   after do
-    MYSQL_DB.drop_table(:items)
+    MYSQL_DB.drop_table?(:items)
   end
 
   specify "#insert should insert record with default values when no arguments given" do
@@ -786,6 +788,62 @@ describe "MySQL::Dataset#insert and related methods" do
     @d.all.should == [{:name => 'abc', :value => 6}, {:name => 'def', :value => 2}]
   end
 
+  specify "#multi_replace should insert multiple records in a single statement" do
+    @d.multi_replace([{:name => 'abc'}, {:name => 'def'}])
+
+    MYSQL_DB.sqls.should == [
+      SQL_BEGIN,
+      "REPLACE INTO `items` (`name`) VALUES ('abc'), ('def')",
+      SQL_COMMIT
+    ]
+
+    @d.all.should == [
+      {:name => 'abc', :value => nil}, {:name => 'def', :value => nil}
+    ]
+  end
+
+  specify "#multi_replace should split the list of records into batches if :commit_every option is given" do
+    @d.multi_replace([{:value => 1}, {:value => 2}, {:value => 3}, {:value => 4}],
+      :commit_every => 2)
+
+    MYSQL_DB.sqls.should == [
+      SQL_BEGIN,
+      "REPLACE INTO `items` (`value`) VALUES (1), (2)",
+      SQL_COMMIT,
+      SQL_BEGIN,
+      "REPLACE INTO `items` (`value`) VALUES (3), (4)",
+      SQL_COMMIT
+    ]
+
+    @d.all.should == [
+      {:name => nil, :value => 1}, 
+      {:name => nil, :value => 2},
+      {:name => nil, :value => 3}, 
+      {:name => nil, :value => 4}
+    ]
+  end
+
+  specify "#multi_replace should split the list of records into batches if :slice option is given" do
+    @d.multi_replace([{:value => 1}, {:value => 2}, {:value => 3}, {:value => 4}],
+      :slice => 2)
+
+    MYSQL_DB.sqls.should == [
+      SQL_BEGIN,
+      "REPLACE INTO `items` (`value`) VALUES (1), (2)",
+      SQL_COMMIT,
+      SQL_BEGIN,
+      "REPLACE INTO `items` (`value`) VALUES (3), (4)",
+      SQL_COMMIT
+    ]
+
+    @d.all.should == [
+      {:name => nil, :value => 1}, 
+      {:name => nil, :value => 2},
+      {:name => nil, :value => 3}, 
+      {:name => nil, :value => 4}
+    ]
+  end
+
   specify "#multi_insert should insert multiple records in a single statement" do
     @d.multi_insert([{:name => 'abc'}, {:name => 'def'}])
 
@@ -916,7 +974,7 @@ describe "MySQL::Dataset#update and related methods" do
     @d = MYSQL_DB[:items]
   end
   after do
-    MYSQL_DB.drop_table(:items)
+    MYSQL_DB.drop_table?(:items)
   end
 
   specify "#update_ignore should not raise error where normal update would fail" do
@@ -937,7 +995,7 @@ describe "MySQL::Dataset#replace" do
     MYSQL_DB.sqls.clear
   end
   after do
-    MYSQL_DB.drop_table(:items)
+    MYSQL_DB.drop_table?(:items)
   end
 
   specify "should use default values if they exist" do
@@ -1005,7 +1063,7 @@ describe "MySQL::Dataset#calc_found_rows" do
     MYSQL_DB.create_table!(:items){Integer :a}
   end
   after do
-    MYSQL_DB.drop_table(:items)
+    MYSQL_DB.drop_table?(:items)
   end
 
   specify "should add the SQL_CALC_FOUND_ROWS keyword when selecting" do
@@ -1035,7 +1093,7 @@ if MYSQL_DB.adapter_scheme == :mysql or MYSQL_DB.adapter_scheme == :jdbc or MYSQ
       MYSQL_DB.sqls.clear
     end
     after do
-      MYSQL_DB.drop_table(:items)
+      MYSQL_DB.drop_table?(:items)
       MYSQL_DB.execute('DROP PROCEDURE test_sproc')
     end
 
@@ -1126,7 +1184,7 @@ if MYSQL_DB.adapter_scheme == :mysql
       MYSQL_DB[:b].insert(25)
     end
     after do
-      MYSQL_DB.drop_table(:a, :b)
+      MYSQL_DB.drop_table?(:a, :b)
     end
 
     specify "should combine all results by default" do
