@@ -193,8 +193,7 @@ describe "A new Database" do
   end
 
   specify "should populate :adapter option when using connection string" do
-    Sequel::Database.should_receive(:adapter_class).once.with(:do).and_return(Sequel::Database)
-    Sequel.connect('do:test://host/db_name').opts[:adapter] == :do
+    Sequel.connect('mock:/').opts[:adapter].should == "mock"
   end
 end
 
@@ -325,6 +324,10 @@ describe "Database#uri" do
     @db.uri.should == 'mau://user:pass@localhost:9876/maumau'
   end
   
+  specify "should return nil if a connection uri was not used" do
+    Sequel.mock.uri.should be_nil
+  end
+  
   specify "should be aliased as #url" do
     @db.url.should == 'mau://user:pass@localhost:9876/maumau'
   end
@@ -413,6 +416,13 @@ describe "Database#extend_datasets" do
     @db.extend_datasets(@m)
   end
   
+  specify "should clear a cached dataset" do
+    @db = Sequel::Database.new
+    @db.literal(1).should == '1'
+    @db.extend_datasets{def literal(v) '2' end}
+    @db.literal(1).should == '2'
+  end
+
   specify "should change the dataset class to a subclass the first time it is called" do
     @db.dataset_class.superclass.should == Sequel::Dataset
   end
@@ -484,6 +494,12 @@ describe "Database#indexes" do
   end
 end
 
+describe "Database#foreign_key_list" do
+  specify "should raise Sequel::NotImplemented" do
+    proc {Sequel::Database.new.foreign_key_list(:table)}.should raise_error(Sequel::NotImplemented)
+  end
+end
+
 describe "Database#run" do
   before do
     @db = Sequel.mock(:servers=>{:s1=>{}})
@@ -620,6 +636,45 @@ shared_examples_for "Database#transaction" do
     proc {@db.transaction {raise RuntimeError}}.should raise_error(RuntimeError)
   end
   
+  specify "should handle errors when sending BEGIN" do
+    ec = Class.new(StandardError)
+    @db.meta_def(:database_error_classes){[ec]}
+    @db.meta_def(:log_connection_execute){|c, sql| sql =~ /BEGIN/ ? raise(ec, 'bad') : super(c, sql)}
+    begin
+      @db.transaction{@db.execute 'DROP TABLE test;'}
+    rescue Sequel::DatabaseError => e
+    end
+    e.should_not be_nil
+    e.wrapped_exception.should be_a_kind_of(ec)
+    @db.sqls.should == ['ROLLBACK']
+  end
+  
+  specify "should handle errors when sending COMMIT" do
+    ec = Class.new(StandardError)
+    @db.meta_def(:database_error_classes){[ec]}
+    @db.meta_def(:log_connection_execute){|c, sql| sql =~ /COMMIT/ ? raise(ec, 'bad') : super(c, sql)}
+    begin
+      @db.transaction{@db.execute 'DROP TABLE test;'}
+    rescue Sequel::DatabaseError => e
+    end
+    e.should_not be_nil
+    e.wrapped_exception.should be_a_kind_of(ec)
+    @db.sqls.should == ['BEGIN', 'DROP TABLE test;']
+  end
+  
+  specify "should handle errors when sending ROLLBACK" do
+    ec = Class.new(StandardError)
+    @db.meta_def(:database_error_classes){[ec]}
+    @db.meta_def(:log_connection_execute){|c, sql| sql =~ /ROLLBACK/ ? raise(ec, 'bad') : super(c, sql)}
+    begin
+      @db.transaction{raise ArgumentError, 'asdf'}
+    rescue Sequel::DatabaseError => e
+    end
+    e.should_not be_nil
+    e.wrapped_exception.should be_a_kind_of(ec)
+    @db.sqls.should == ['BEGIN']
+  end
+  
   specify "should issue ROLLBACK if Sequel::Rollback is called in the transaction" do
     @db.transaction do
       @db.drop_table(:a)
@@ -1061,21 +1116,20 @@ describe "A Database adapter with a scheme" do
     proc {Sequel.ccc('abc', 'def')}.should raise_error(Sequel::Error)
     
     c = Sequel.ccc('mydb')
-    p = proc{c.opts.delete_if{|k,v| k == :disconnection_proc || k == :single_threaded}}
     c.should be_a_kind_of(@ccc)
-    p.call.should == {:adapter=>:ccc, :database => 'mydb', :adapter_class=>@ccc}
+    c.opts.values_at(:adapter, :database, :adapter_class).should == [:ccc, 'mydb', @ccc]
     
     c = Sequel.ccc('mydb', :host => 'localhost')
     c.should be_a_kind_of(@ccc)
-    p.call.should == {:adapter=>:ccc, :database => 'mydb', :host => 'localhost', :adapter_class=>@ccc}
+    c.opts.values_at(:adapter, :database, :host, :adapter_class).should == [:ccc, 'mydb', 'localhost', @ccc]
     
     c = Sequel.ccc
     c.should be_a_kind_of(@ccc)
-    p.call.should == {:adapter=>:ccc, :adapter_class=>@ccc}
+    c.opts.values_at(:adapter, :adapter_class).should == [:ccc, @ccc]
     
     c = Sequel.ccc(:database => 'mydb', :host => 'localhost')
     c.should be_a_kind_of(@ccc)
-    p.call.should == {:adapter=>:ccc, :database => 'mydb', :host => 'localhost', :adapter_class=>@ccc}
+    c.opts.values_at(:adapter, :database, :host, :adapter_class).should == [:ccc, 'mydb', 'localhost', @ccc]
   end
   
   specify "should be accessible through Sequel.connect with options" do
@@ -1336,6 +1390,14 @@ describe "Database#inspect" do
   specify "should include the class name and the connection url" do
     Sequel.connect('mock://foo/bar').inspect.should == '#<Sequel::Mock::Database: "mock://foo/bar">'
   end
+
+  specify "should include the class name and the connection options if an options hash was given" do
+    Sequel.connect(:adapter=>:mock).inspect.should =~ /#<Sequel::Mock::Database: \{:adapter=>:mock\}>/
+  end
+
+  specify "should include the class name, uri, and connection options if uri and options hash was given" do
+    Sequel.connect('mock://foo', :database=>'bar').inspect.should =~ /#<Sequel::Mock::Database: "mock:\/\/foo" \{:database=>"bar"\}>/
+  end
 end
 
 describe "Database#get" do
@@ -1928,12 +1990,37 @@ describe "Database#schema_autoincrementing_primary_key?" do
   end
 end
 
+describe "Database#supports_transactional_ddl?" do
+  specify "should be false by default" do
+    Sequel::Database.new.supports_transactional_ddl?.should == false
+  end
+end
+
+describe "Database#global_index_namespace?" do
+  specify "should be true by default" do
+    Sequel::Database.new.global_index_namespace?.should == true
+  end
+end
+
 describe "Database#supports_savepoints?" do
   specify "should be false by default" do
     Sequel::Database.new.supports_savepoints?.should == false
   end
 end
 
+describe "Database#supports_savepoints_in_prepared_transactions?" do
+  specify "should be false by default" do
+    Sequel::Database.new.supports_savepoints_in_prepared_transactions?.should == false
+  end
+
+  specify "should be true if both savepoints and prepared transactions are supported" do
+    db = Sequel::Database.new
+    db.meta_def(:supports_savepoints?){true}
+    db.meta_def(:supports_prepared_transactions?){true}
+    db.supports_savepoints_in_prepared_transactions?.should == true
+  end
+end
+
 describe "Database#supports_prepared_transactions?" do
   specify "should be false by default" do
     Sequel::Database.new.supports_prepared_transactions?.should == false
