@@ -536,7 +536,7 @@ end
 describe "Sequel::Model Simple Associations" do
   before(:all) do
     @db = INTEGRATION_DB
-    [:albums_tags, :tags, :albums, :artists].each{|t| @db.drop_table(t) rescue nil}
+    @db.drop_table?(:albums_tags, :tags, :albums, :artists)
     @db.create_table(:artists) do
       primary_key :id
       String :name
@@ -598,7 +598,7 @@ describe "Sequel::Model Simple Associations" do
     [:Tag, :Album, :Artist].each{|x| Object.send(:remove_const, x)}
   end
   after(:all) do
-    @db.drop_table(:albums_tags, :tags, :albums, :artists)
+    @db.drop_table?(:albums_tags, :tags, :albums, :artists)
   end
   
   it_should_behave_like "regular and composite key associations"
@@ -608,7 +608,7 @@ describe "Sequel::Model Simple Associations" do
       @els = {:eager_limit_strategy=>:correlated_subquery}
     end
     it_should_behave_like "eager limit strategies"
-  end unless Sequel.guarded?(:mysql, :db2, :oracle)
+  end unless Sequel.guarded?(:mysql, :db2, :oracle, :h2)
 
   specify "should handle many_to_one associations with same name as :key" do
     Album.def_column_alias(:artist_id_id, :artist_id)
@@ -693,12 +693,10 @@ describe "Sequel::Model Simple Associations" do
     @album.artist(proc{|ds| ds.exclude(:id=>@artist.id)}).should == nil
     @album.artist(proc{|ds| ds.filter(:id=>@artist.id)}).should == @artist
 
-    if RUBY_VERSION >= '1.8.7'
-      @artist.albums{|ds| ds.exclude(:id=>@album.id)}.should == []
-      @artist.albums{|ds| ds.filter(:id=>@album.id)}.should == [@album]
-      @album.artist{|ds| ds.exclude(:id=>@artist.id)}.should == nil
-      @album.artist{|ds| ds.filter(:id=>@artist.id)}.should == @artist
-    end
+    @artist.albums{|ds| ds.exclude(:id=>@album.id)}.should == []
+    @artist.albums{|ds| ds.filter(:id=>@album.id)}.should == [@album]
+    @album.artist{|ds| ds.exclude(:id=>@artist.id)}.should == nil
+    @album.artist{|ds| ds.filter(:id=>@artist.id)}.should == @artist
   end
   
   specify "should handle dynamic callbacks for eager loading via eager and eager_graph" do
@@ -733,7 +731,7 @@ end
 describe "Sequel::Model Composite Key Associations" do
   before(:all) do
     @db = INTEGRATION_DB
-    [:albums_tags, :tags, :albums, :artists].each{|t| @db.drop_table(t) rescue nil}
+    @db.drop_table?(:albums_tags, :tags, :albums, :artists)
     @db.create_table(:artists) do
       Integer :id1
       Integer :id2
@@ -813,7 +811,7 @@ describe "Sequel::Model Composite Key Associations" do
     [:Tag, :Album, :Artist].each{|x| Object.send(:remove_const, x)}
   end
   after(:all) do
-    @db.drop_table(:albums_tags, :tags, :albums, :artists)
+    @db.drop_table?(:albums_tags, :tags, :albums, :artists)
   end
 
   it_should_behave_like "regular and composite key associations"
@@ -868,7 +866,7 @@ end
 describe "Sequel::Model Associations with clashing column names" do
   before(:all) do
     @db = INTEGRATION_DB
-    [:bars_foos, :bars, :foos].each{|t| @db.drop_table(t) rescue nil}
+    @db.drop_table?(:bars_foos, :bars, :foos)
     @db.create_table(:foos) do
       primary_key :id
       Integer :object_id
@@ -899,7 +897,7 @@ describe "Sequel::Model Associations with clashing column names" do
     @Foo.db[:bars_foos].insert(2, 2)
   end
   after(:all) do
-    @db.drop_table(:bars_foos, :bars, :foos)
+    @db.drop_table?(:bars_foos, :bars, :foos)
   end
 
   it "should have working regular association methods" do
