@@ -11,7 +11,7 @@ describe "Simple Dataset operations" do
     @ds.insert(:number=>10)
   end
   after do
-    @db.drop_table(:items)
+    @db.drop_table?(:items)
   end
 
   specify "should support sequential primary keys" do
@@ -23,6 +23,16 @@ describe "Simple Dataset operations" do
       {:id => 3, :number=>30} ]   
   end 
 
+  specify "should support sequential primary keys with a Bignum" do
+    @db.create_table!(:items) do
+      primary_key :id, :type=>Bignum
+      Integer :number
+    end
+    @ds << {:number=>20}
+    @ds << {:number=>30}
+    @ds.order(:number).all.should == [{:id => 1, :number=>20}, {:id => 2, :number=>30}]   
+  end 
+
   cspecify "should insert with a primary key specified", :db2, :mssql do
     @ds.insert(:id=>100, :number=>20)
     @ds.count.should == 2
@@ -172,6 +182,33 @@ describe "Simple Dataset operations" do
   end
 end
 
+describe "Simple dataset operations with nasty table names" do
+  before do
+    @db = INTEGRATION_DB
+    @table = :"i`t' [e]\"m\\s" 
+    @qi = @db.quote_identifiers?
+    @db.quote_identifiers = true
+  end
+  after do
+    @db.quote_identifiers = @qi
+  end
+
+  cspecify "should work correctly", :mssql, :oracle do
+    @db.create_table!(@table) do
+      primary_key :id
+      Integer :number
+    end
+    @ds = @db[@table]
+    @ds.insert(:number=>10).should == 1
+    @ds.all.should == [{:id=>1, :number=>10}]
+    @ds.update(:number=>20).should == 1 
+    @ds.all.should == [{:id=>1, :number=>20}]
+    @ds.delete.should == 1
+    @ds.count.should == 0
+    proc{@db.drop_table?(@table)}.should_not raise_error
+  end 
+end
+
 describe Sequel::Dataset do
   before do
     INTEGRATION_DB.create_table!(:test) do
@@ -181,7 +218,7 @@ describe Sequel::Dataset do
     @d = INTEGRATION_DB[:test]
   end
   after do
-    INTEGRATION_DB.drop_table(:test)
+    INTEGRATION_DB.drop_table?(:test)
   end
 
   specify "should return the correct record count" do
@@ -254,20 +291,31 @@ end
 
 describe Sequel::Database do
   specify "should correctly escape strings" do
-    INTEGRATION_DB.get("\\dingo".as(:a)) == "\\dingo"
+    ["\\\n",
+     "\\\\\n",
+     "\\\r\n",
+     "\\\\\r\n",
+     "\\\\\n\n", 
+     "\\\\\r\n\r\n",
+     "\\dingo",
+     "\\'dingo",
+     "\\\\''dingo",
+    ].each do |str|
+      INTEGRATION_DB.get(str.as(:a)).should == str
+      str = "1#{str}1"
+      INTEGRATION_DB.get(str.as(:a)).should == str
+      str = "#{str}#{str}"
+      INTEGRATION_DB.get(str.as(:a)).should == str
+    end
   end
 
-  specify "should correctly escape strings with quotes" do
-    INTEGRATION_DB.get("\\'dingo".as(:a)) == "\\'dingo"
-  end
-
-  specify "should properly escape binary data" do
-    INTEGRATION_DB.get("\1\2\3".to_sequel_blob.as(:a)) == "\1\2\3"
+  cspecify "should properly escape binary data", [:odbc], [:jdbc, :hsqldb], [:swift], :oracle do
+    INTEGRATION_DB.get("\1\2\3".to_sequel_blob.cast(File).as(:a)).should == "\1\2\3"
   end
 
   specify "should have a working table_exists?" do
     t = :basdfdsafsaddsaf
-    INTEGRATION_DB.drop_table(t) rescue nil
+    INTEGRATION_DB.drop_table?(t)
     INTEGRATION_DB.table_exists?(t).should == false
     INTEGRATION_DB.create_table(t){Integer :a}
     begin
@@ -290,7 +338,7 @@ describe Sequel::Dataset do
     @d << {:value => 789}
   end 
   after do
-    INTEGRATION_DB.drop_table(:items)
+    INTEGRATION_DB.drop_table?(:items)
   end 
   
   specify "should correctly return avg" do
@@ -319,7 +367,7 @@ describe "Simple Dataset operations" do
     @ds = INTEGRATION_DB[:items]
   end
   after do
-    INTEGRATION_DB.drop_table(:items)
+    INTEGRATION_DB.drop_table?(:items)
   end
 
   specify "should deal with boolean conditions correctly" do
@@ -345,7 +393,7 @@ describe "Simple Dataset operations in transactions" do
     @ds = INTEGRATION_DB[:items]
   end
   after do
-    INTEGRATION_DB.drop_table(:items)
+    INTEGRATION_DB.drop_table?(:items)
   end
 
   cspecify "should insert correctly with a primary key specified inside a transaction", :db2, :mssql do
@@ -456,7 +504,7 @@ if INTEGRATION_DB.dataset.supports_cte?
       @ds.insert(:id=>6, :parent_id=>5)
     end
     after(:all) do
-      @db.drop_table(:i1)
+      @db.drop_table?(:i1)
     end
     
     specify "should give correct results for WITH" do
@@ -508,7 +556,7 @@ if INTEGRATION_DB.dataset.supports_cte?(:update) # Assume INSERT and DELETE supp
       @ds.insert(:id=>2)
     end
     after do
-      @db.drop_table(:i1)
+      @db.drop_table?(:i1)
     end
     
     specify "should give correct results for WITH" do
@@ -530,7 +578,7 @@ if INTEGRATION_DB.dataset.supports_returning?(:insert)
       @ds = @db[:i1]
     end
     after do
-      @db.drop_table(:i1)
+      @db.drop_table?(:i1)
     end
     
     specify "should give correct results" do
@@ -556,7 +604,7 @@ if INTEGRATION_DB.dataset.supports_returning?(:update) # Assume DELETE support a
       @ds.insert(1, 2)
     end
     after do
-      @db.drop_table(:i1)
+      @db.drop_table?(:i1)
     end
     
     specify "should give correct results" do
@@ -597,7 +645,7 @@ if INTEGRATION_DB.dataset.supports_window_functions?
       @ds.insert(:id=>6, :group_id=>2, :amount=>100000)
     end
     after(:all) do
-      @db.drop_table(:i1)
+      @db.drop_table?(:i1)
     end
     
     specify "should give correct results for aggregate window functions" do
@@ -652,7 +700,7 @@ describe Sequel::SQL::Constants do
     @c2 = proc{|v| v.is_a?(Date) ? v : Date.parse(v) }
   end
   after do
-    @db.drop_table(:constants) if @db.table_exists?(:constants)
+    @db.drop_table?(:constants)
   end
   
   cspecify "should have working CURRENT_DATE", [:odbc, :mssql], [:jdbc, :sqlite], :oracle do
@@ -669,13 +717,13 @@ describe Sequel::SQL::Constants do
     (Time.now - @c[@ds.get(:t)]).should be_within(2).of(0)
   end
 
-  cspecify "should have working CURRENT_TIMESTAMP", [:jdbc, :sqlite] do
+  cspecify "should have working CURRENT_TIMESTAMP", [:jdbc, :sqlite], [:swift] do
     @db.create_table!(:constants){DateTime :ts}
     @ds.insert(:ts=>Sequel::CURRENT_TIMESTAMP)
     (Time.now - @c[@ds.get(:ts)]).should be_within(2).of(0)
   end
 
-  cspecify "should have working CURRENT_TIMESTAMP when used as a column default", [:jdbc, :sqlite] do
+  cspecify "should have working CURRENT_TIMESTAMP when used as a column default", [:jdbc, :sqlite], [:swift] do
     @db.create_table!(:constants){DateTime :ts, :default=>Sequel::CURRENT_TIMESTAMP}
     @ds.insert
     (Time.now - @c[@ds.get(:ts)]).should be_within(2).of(0)
@@ -692,7 +740,7 @@ describe "Sequel::Dataset#import and #multi_insert" do
     @ids.delete
   end
   after(:all) do
-    @db.drop_table(:imp)
+    @db.drop_table?(:imp)
   end
 
   it "should import with multi_insert and an array of hashes" do
@@ -732,7 +780,7 @@ describe "Sequel::Dataset#import and #multi_insert :return=>:primary_key " do
     @ds = @db[:imp]
   end
   after do
-    @db.drop_table(:imp)
+    @db.drop_table?(:imp)
   end
 
   specify "should return primary key values " do
@@ -760,7 +808,7 @@ describe "Sequel::Dataset convenience methods" do
     @ds.insert(2, 4, 6)
   end
   after(:all) do
-    @db.drop_table(:a)
+    @db.drop_table?(:a)
   end
   
   it "#group_rollup should include hierarchy of groupings" do
@@ -784,7 +832,7 @@ describe "Sequel::Dataset convenience methods" do
     @ds.delete
   end
   after(:all) do
-    @db.drop_table(:a)
+    @db.drop_table?(:a)
   end
   
   it "#[]= should update matching rows" do
@@ -862,7 +910,7 @@ describe "Sequel::Dataset main SQL methods" do
     @ds.delete
   end
   after(:all) do
-    @db.drop_table(:a)
+    @db.drop_table?(:a)
   end
   
   it "#exists should return a usable exists clause" do
@@ -935,7 +983,7 @@ describe "Sequel::Dataset convenience methods" do
     @ds.insert(5, 6, 7, 8)
   end
   after(:all) do
-    @db.drop_table(:a)
+    @db.drop_table?(:a)
   end
   
   specify "should have working #map" do
@@ -955,6 +1003,19 @@ describe "Sequel::Dataset convenience methods" do
     @ds.to_hash([:a, :c], [:b, :d]).should == {[1, 3]=>[2, 4], [5, 7]=>[6, 8]}
   end
 
+  specify "should have working #to_hash_groups" do
+    ds = @ds.order(*@ds.columns)
+    ds.insert(1, 2, 3, 9)
+    ds.to_hash_groups(:a).should == {1=>[{:a=>1, :b=>2, :c=>3, :d=>4}, {:a=>1, :b=>2, :c=>3, :d=>9}], 5=>[{:a=>5, :b=>6, :c=>7, :d=>8}]}
+    ds.to_hash_groups(:b).should == {2=>[{:a=>1, :b=>2, :c=>3, :d=>4}, {:a=>1, :b=>2, :c=>3, :d=>9}], 6=>[{:a=>5, :b=>6, :c=>7, :d=>8}]}
+    ds.to_hash_groups([:a, :b]).should == {[1, 2]=>[{:a=>1, :b=>2, :c=>3, :d=>4}, {:a=>1, :b=>2, :c=>3, :d=>9}], [5, 6]=>[{:a=>5, :b=>6, :c=>7, :d=>8}]}
+
+    ds.to_hash_groups(:a, :d).should == {1=>[4, 9], 5=>[8]}
+    ds.to_hash_groups([:a, :c], :d).should == {[1, 3]=>[4, 9], [5, 7]=>[8]}
+    ds.to_hash_groups(:a, [:b, :d]).should == {1=>[[2, 4], [2, 9]], 5=>[[6, 8]]}
+    ds.to_hash_groups([:a, :c], [:b, :d]).should == {[1, 3]=>[[2, 4], [2, 9]], [5, 7]=>[[6, 8]]}
+  end
+
   specify "should have working #select_map" do
     @ds.select_map(:a).should == [1, 5]
     @ds.select_map(:b).should == [2, 6]
@@ -997,6 +1058,19 @@ describe "Sequel::Dataset convenience methods" do
     @ds.select_hash(:a, [:b, :c]).should == {1=>[2, 3], 5=>[6, 7]}
     @ds.select_hash([:a, :c], [:b, :d]).should == {[1, 3]=>[2, 4], [5, 7]=>[6, 8]}
   end
+
+  specify "should have working #select_hash_groups" do
+    ds = @ds.order(*@ds.columns)
+    ds.insert(1, 2, 3, 9)
+    ds.select_hash_groups(:a, :d).should == {1=>[4, 9], 5=>[8]}
+    ds.select_hash_groups(:a__a___e, :d).should == {1=>[4, 9], 5=>[8]}
+    ds.select_hash_groups(:a__a.as(:e), :d).should == {1=>[4, 9], 5=>[8]}
+    ds.select_hash_groups(:a.qualify(:a).as(:e), :d).should == {1=>[4, 9], 5=>[8]}
+    ds.select_hash_groups(:a.identifier.qualify(:a).as(:e), :d).should == {1=>[4, 9], 5=>[8]}
+    ds.select_hash_groups([:a, :c], :d).should == {[1, 3]=>[4, 9], [5, 7]=>[8]}
+    ds.select_hash_groups(:a, [:b, :d]).should == {1=>[[2, 4], [2, 9]], 5=>[[6, 8]]}
+    ds.select_hash_groups([:a, :c], [:b, :d]).should == {[1, 3]=>[[2, 4], [2, 9]], [5, 7]=>[[6, 8]]}
+  end
 end
 
 describe "Sequel::Dataset DSL support" do
@@ -1009,7 +1083,7 @@ describe "Sequel::Dataset DSL support" do
     @ds.delete
   end
   after(:all) do
-    @db.drop_table(:a)
+    @db.drop_table?(:a)
   end
   
   it "should work with standard mathematical operators" do
@@ -1052,6 +1126,13 @@ describe "Sequel::Dataset DSL support" do
     @ds.get{a.sql_number ^ b ^ 1}.to_i.should == 7
   end
   
+  specify "should work with the modulus operator" do
+    @ds.insert(3, 5)
+    @ds.get{a.sql_number % 4}.to_i.should == 3
+    @ds.get{b.sql_number % 4}.to_i.should == 1
+    @ds.get{a.sql_number % 4 % 2}.to_i.should == 1
+  end
+  
   specify "should work with inequality operators" do
     @ds.insert(10, 11)
     @ds.insert(11, 11)
@@ -1237,7 +1318,7 @@ describe "SQL Extract Function" do
     @ds = @db[:a].order(:a)
   end
   after do
-    @db.drop_table(:a)
+    @db.drop_table?(:a)
   end
   
   specify "should return the part of the datetime asked for" do
@@ -1270,7 +1351,7 @@ describe "Dataset string methods" do
     @ds.delete
   end
   after(:all) do
-    @db.drop_table(:a)
+    @db.drop_table?(:a)
   end
   
   it "#grep should return matching rows" do
@@ -1354,7 +1435,7 @@ describe "Dataset identifier methods" do
     @ds = @db[:a].order(:ab)
   end
   after(:all) do
-    @db.drop_table(:a)
+    @db.drop_table?(:a)
   end
   
   cspecify "#identifier_output_method should change how identifiers are output", [:swift] do
@@ -1385,7 +1466,7 @@ describe "Dataset defaults and overrides" do
     @ds.delete
   end
   after(:all) do
-    @db.drop_table(:a)
+    @db.drop_table?(:a)
   end
   
   it "#set_defaults should set defaults that can be overridden" do
@@ -1419,7 +1500,7 @@ if INTEGRATION_DB.dataset.supports_modifying_joins?
       @db[:c].insert(7, 8)
     end
     after do
-      @db.drop_table(:a, :b, :c)
+      @db.drop_table?(:a, :b, :c)
     end
     
     it "#update should allow updating joined datasets" do
