@@ -9,6 +9,12 @@ describe "Model attribute setters" do
     MODEL_DB.reset
   end
 
+  specify "refresh should return self" do
+    @o = @c[1]
+    @o.stub(:_refresh).and_return([])
+    @o.refresh.should == @o
+  end
+
   it "should mark the column value as changed" do
     @o.changed_columns.should == []
 
@@ -349,6 +355,54 @@ describe "Model.qualified_primary_key_hash" do
   end
 end
 
+describe "Model.db" do
+  before do
+    @db = Sequel.mock
+    @databases = Sequel::DATABASES.dup
+    @model_db = Sequel::Model.db
+    Sequel::Model.db = nil
+    Sequel::DATABASES.clear
+  end
+  after do
+    Sequel::Model.instance_variable_get(:@db).should == nil
+    Sequel::DATABASES.replace(@databases)
+    Sequel::Model.db = @model_db
+  end
+
+  specify "should be required when create named model classes" do
+    begin
+      proc{class ModelTest < Sequel::Model; end}.should raise_error(Sequel::Error)
+    ensure
+      Object.send(:remove_const, :ModelTest)
+    end
+  end
+
+  specify "should be required when creating anonymous model classes without a database" do
+    proc{Sequel::Model(:foo)}.should raise_error(Sequel::Error)
+  end
+
+  specify "should not be required when creating anonymous model classes with a database" do
+    Sequel::Model(@db).db.should == @db
+    Sequel::Model(@db[:foo]).db.should == @db
+  end
+
+  specify "should work correctly when subclassing anonymous model classes with a database" do
+    begin
+      Class.new(Sequel::Model(@db)).db.should == @db
+      Class.new(Sequel::Model(@db[:foo])).db.should == @db
+      class ModelTest < Sequel::Model(@db)
+        db.should == @db
+      end
+      class ModelTest2 < Sequel::Model(@db[:foo])
+        db.should == @db
+      end
+    ensure
+      Object.send(:remove_const, :ModelTest)
+      Object.send(:remove_const, :ModelTest2)
+    end
+  end
+end
+
 describe "Model.db=" do
   before do
     @db1 = Sequel.mock
@@ -407,7 +461,7 @@ describe Sequel::Model, ".(allowed|restricted)_columns " do
     i.set(:x => 4, :y => 5, :z => 6)
     i.values.should == {:x => 4, :y => 5}
 
-    @c.dataset._fetch = {:x => 7}
+    @c.instance_dataset._fetch = @c.dataset._fetch = {:x => 7}
     i = @c.new
     i.update(:x => 7, :z => 9)
     i.values.should == {:x => 7}
@@ -421,7 +475,7 @@ describe Sequel::Model, ".(allowed|restricted)_columns " do
     i.set(:x => 4, :y => 5, :z => 6)
     i.values.should == {:x => 4, :y => 5}
 
-    @c.dataset._fetch = {:x => 7}
+    @c.instance_dataset._fetch = @c.dataset._fetch = {:x => 7}
     i = @c.new
     i.update(:x => 7, :z => 9)
     i.values.should == {:x => 7}
@@ -436,7 +490,7 @@ describe Sequel::Model, ".(allowed|restricted)_columns " do
     i.set(:x => 4, :y => 5, :z => 6)
     i.values.should == {:x => 4, :y => 5}
 
-    @c.dataset._fetch = {:y => 7}
+    @c.instance_dataset._fetch = @c.dataset._fetch = {:y => 7}
     i = @c.new
     i.update(:y => 7, :z => 9)
     i.values.should == {:y => 7}
@@ -610,7 +664,7 @@ describe Sequel::Model, ".[] optimization" do
 
   it "should use Dataset#with_sql if simple_table and simple_pk are true" do
     @c.set_dataset :a
-    @c.dataset._fetch = {:id => 1}
+    @c.instance_dataset._fetch = @c.dataset._fetch = {:id => 1}
     @c[1].should == @c.load(:id=>1)
     @db.sqls.should == ['SELECT * FROM "a" WHERE "id" = 1']
   end
