@@ -67,7 +67,11 @@ describe "Sequel::Model()" do
   end
 
   describe "reloading" do
+    before do
+      Sequel::Model.cache_anonymous_models = true
+    end
     after do
+      Sequel::Model.cache_anonymous_models = false
       Object.send(:remove_const, :Album) if defined?(::Album)
     end
 
@@ -126,6 +130,14 @@ describe "Sequel::Model()" do
         class ::Album < Sequel::Model(@db[:table.identifier]); end
       end.should_not raise_error
     end
+
+    it "should raise an exception if anonymous model caching is disabled" do
+      Sequel::Model.cache_anonymous_models = false
+      proc do
+        class ::Album < Sequel::Model(@db[:table.identifier]); end
+        class ::Album < Sequel::Model(@db[:table.identifier]); end
+      end.should raise_error
+    end
   end
 end
 
@@ -222,7 +234,7 @@ describe Sequel::Model, "dataset & schema" do
     MODEL_DB.sqls
     ds._fetch = [{:id=>1}, {:id=>2}]
     ds.destroy.should == 2
-    MODEL_DB.sqls.should == ["SELECT * FROM foo", "DELETE FROM foo WHERE (id = 1)", "DELETE FROM foo WHERE (id = 2)"]
+    MODEL_DB.sqls.should == ["SELECT * FROM foo", "DELETE FROM foo WHERE id = 1", "DELETE FROM foo WHERE id = 2"]
   end
 
   it "set_dataset should add the destroy method that respects sharding with transactions" do
@@ -327,8 +339,8 @@ describe Sequel::Model, "new" do
   end
 
   it "should use the last inserted id as primary key if not in values" do
-    @m.dataset._fetch = {:x => 1, :id => 1234}
-    @m.dataset.autoid = 1234
+    @m.instance_dataset._fetch = @m.dataset._fetch = {:x => 1, :id => 1234}
+    @m.instance_dataset.autoid = @m.dataset.autoid = 1234
 
     o = @m.new(:x => 1)
     o.save
@@ -426,8 +438,8 @@ describe Sequel::Model, ".find_or_create" do
   end
   
   it "should create the record if not found" do
-    @c.dataset._fetch = [[], {:x=>1, :id=>1}]
-    @c.dataset.autoid = 1
+    @c.instance_dataset._fetch = @c.dataset._fetch = [[], {:x=>1, :id=>1}]
+    @c.instance_dataset.autoid = @c.dataset.autoid = 1
     @c.find_or_create(:x => 1).should == @c.load(:x=>1, :id=>1)
     MODEL_DB.sqls.should == ["SELECT * FROM items WHERE (x = 1) LIMIT 1",
       "INSERT INTO items (x) VALUES (1)",
@@ -435,8 +447,8 @@ describe Sequel::Model, ".find_or_create" do
   end
 
   it "should pass the new record to be created to the block if no record is found" do
-    @c.dataset._fetch = [[], {:x=>1, :id=>1}]
-    @c.dataset.autoid = 1
+    @c.instance_dataset._fetch = @c.dataset._fetch = [[], {:x=>1, :id=>1}]
+    @c.instance_dataset.autoid = @c.dataset.autoid = 1
     @c.find_or_create(:x => 1){|x| x[:y] = 2}.should == @c.load(:x=>1, :id=>1)
     sqls = MODEL_DB.sqls
     sqls.first.should == "SELECT * FROM items WHERE (x = 1) LIMIT 1"
@@ -559,6 +571,12 @@ describe Sequel::Model, ".[]" do
     MODEL_DB.sqls.should == ["SELECT * FROM items WHERE id = 9999"]
   end
 
+  it "should have #[] return nil if no rows match" do
+    @c.dataset._fetch = []
+    @c[1].should == nil
+    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE id = 1"]
+  end
+
   it "should work correctly for custom primary key" do
     @c.set_primary_key :name
     @c['sharon'].should == @c.load(:name => 'sharon', :id => 1)
